import argparse

from pyreess import DESCRIPTION
from pyreess.generator import Generator

import pyperclip


LENGTH = "LENGTH"

DIGITS = "DIGITS"
LOWERCASE = "LOWERCASE"
UPPERCASE = "UPPERCASE"
SYMBOLS = "SYMBOLS"

SHOW = "SHOW"


def ask(message: str, pos: str = "y", neg: str = "n"):
    while True:
        resp = input(message + " [y/n] ").lower()
        if resp == pos:
            return True
        elif resp == neg:
            return False


def main():
    parser = argparse.ArgumentParser(description=DESCRIPTION)
    parser.add_argument("-len", type=int, default=15, help="Output length (default: %(default)s)", dest=LENGTH)
    parser.add_argument("-sh", type=int, default=3,
                        help="Length of shown output. If negative, output will be fully shown (default: %(default)s)", dest=SHOW)

    alphabet_group = parser.add_argument_group("Alphabet settings")
    alphabet_group.add_argument("-d", action="store_true", help="Use digits", dest=DIGITS)
    alphabet_group.add_argument("-l", action="store_true", help="Use lowercase", dest=LOWERCASE)
    alphabet_group.add_argument("-u", action="store_true", help="Use uppercase", dest=UPPERCASE)
    alphabet_group.add_argument("-s", action="store_true", help="Use symbols", dest=SYMBOLS)

    args = parser.parse_args()

    ################################################################
    length = getattr(args, LENGTH)
    show = getattr(args, SHOW)
    if show > length or show < 0:
        show = length

    digits = getattr(args, DIGITS)
    lowercase = getattr(args, LOWERCASE)
    uppercase = getattr(args, UPPERCASE)
    symbols = getattr(args, SYMBOLS)
    if not (digits or lowercase or uppercase or symbols):
        parser.print_help()
        parser.error("At least one alphabet related flag in required!")

    ################################################################
    source = input("Input string: ")
    if not source:
        print("Empty input string!")
        raise KeyboardInterrupt

    salt = input("Salt: ")
    if not (source or salt):
        print("Empty input string or salt!")
        raise KeyboardInterrupt

    gen = Generator(source=source.encode("utf-8"), salt=salt.encode("utf-8"))
    gen.set_rules(digits=digits,
                  lowercase=lowercase,
                  uppercase=uppercase,
                  symbols=symbols)
    output = gen.generate(length=length)

    ################################################################
    print("----------------")
    print("Output:")
    print(output[:show] + "*" * (len(output) - show))

    print("----------------")
    if ask("Copy output to the clipboard?"):
        pyperclip.copy(output)
        print("Done!")


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass



import string

from seedrandom import SeededRNG


class Generator:
    def __init__(self, source: bytes, salt: bytes):
        self._source = source
        self._salt = salt

        self._alphabet = ""
        self.set_rules()

    def set_rules(self, digits: bool = True, lowercase: bool = True, uppercase: bool = True, symbols: bool = False):
        alphabet = ""

        if digits:
            alphabet += string.digits
        if lowercase:
            alphabet += string.ascii_lowercase
        if uppercase:
            alphabet += string.ascii_uppercase
        if symbols:
            alphabet += "!\"#$%&'()*+,-./:;<=>?@[\\]^_`{|}~"

        self._alphabet = "".join(sorted(alphabet))

    def generate(self, length: int) -> str:
        if length <= 0:
            raise ValueError("Length must be > 0")

        string = ""
        for char_pos in range(length):
            if len(self._salt) > 0:
                salt = [self._salt]
            else:
                salt = []

            # Generate random index for each position and fetch char from alphabet
            rng = SeededRNG(self._source,
                            *salt,
                            self._alphabet.encode("utf-8"),
                            bytes([length]),
                            bytes([char_pos]))  # hashlib.md5 by default

            random_i = rng.randint(a=0, b=len(self._alphabet) - 1)
            string += self._alphabet[random_i]

        return string



# Local
from setuptools import setup, find_packages

# Project
from pyreess import PACKAGE_NAME, __version__, DESCRIPTION


with open("README.md", "r") as file:
    LONG_DESCRIPTION = file.read()

setup(name=PACKAGE_NAME,
      version=__version__,
      install_requires=["seedrandom==2.2", "pyperclip"],
      packages=find_packages(),
      entry_points={
          "console_scripts": [
              "pyreess = pyreess.main:main"
          ]
      },

      # Metadata for PyPi
      author="BananaLoaf",
      author_email="bananaloaf@protonmail.com",
      # maintainer="BananaLoaf",
      # maintainer_email="bananaloaf@protonmail.com",
      license="MIT",

      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      long_description_content_type="text/markdown",
      keywords=["deterministic", "password", "generator"],

      url="https://github.com/BananaLoaf/pyreess",
      # download_url=None,
      project_urls={
          "How secure is my password?": "https://howsecureismypassword.net"
      },

      # https://pypi.org/classifiers/
      classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3"
      ])
